#include <octave/oct.h>

DEFUN_DLD(hysteresis_cpp, args, nargout,
          "-*- texinfo -*-\n"
          "@deftypefn {FunctionFile} {@var{y} = } hysteresis_cpp (@var{on},@var{off},@var{x})\n"
          "\n"
          "Output a counting integer in the vector @var{y} everytime @var{x}\n"
          "crosses the hysteresis set up by @var{on} and @var{off}.\n"
          "\n"
          "@example\n"
          "@group\n"
          "@code{hysteresis_cpp(-1,0,[1,0,1,0,-1,-2,-1,0,1,0,-2,-2,-2])}\n"
          "@result{} [0,0,0,0,1,1,1,1,0,0,2,2,2]\n"
          "@end group\n"
          "@end example\n"
          "@end deftypefn") {
  octave_value_list retval;

  // Check and extract the arguments
  if ( args.length() != 3 ||
       !args(0).is_scalar_type() || !args(1).is_scalar_type() || !args(2).is_matrix_type() || 
       !(args(2).dims().length()==2 && (args(2).dims().elem(0)==1 || args(2).dims().elem(1)==1)) ) {
    print_usage();
    return retval;
  }

  double on  = args(0).scalar_value();
  double off = args(1).scalar_value();
  RowVector x  = args(2).row_vector_value();
  
  // Build the output
  int num = 1;
  int n = x.length();
  RowVector y(n);

  if (on < off)
    for (int i=0; i<n; ++num) {
      // Put 0 as output until we drop below on
      for (; i < n && x(i) >  on ; ++i)
        y(i) = 0;
      // Put num as output until we come above off
      for (; i < n && x(i) <= off; ++i)
        y(i) = num;
    }
  else
    for (int i=0; i<n; ++num) {
      // Put 0 as output until we come above on
      for (; i < n && x(i) <  on ; ++i)
        y(i) = 0;
      // Put num as output until we drop below off
      for (; i < n && x(i) >= off; ++i)
        y(i) = num;
    }

  retval(0) = y;
  return retval;
}
